@@ -1,12 +1,14 @@
 const Cinnamon = imports.gi.Cinnamon;
 const Meta = imports.gi.Meta;
 const Clutter = imports.gi.Clutter;
+const GLib = imports.gi.GLib;
 const St = imports.gi.St;
 const Main = imports.ui.main;
 const Tweener = imports.ui.tweener;
 const DND = imports.ui.dnd;
 const Tooltips = imports.ui.tooltips;
 const PopupMenu = imports.ui.popupMenu;
+const Mainloop = imports.mainloop;
 const {SignalManager} = imports.misc.signalManager;
 const {each, findIndex, unref} = imports.misc.util;
 const {createStore} = imports.misc.state;
@@ -21,6 +23,18 @@ const {
     TitleDisplay
 } = require('./constants');
 
+const _reLetterRtl = new RegExp("\\p{Script=Hebrew}|\\p{Script=Arabic}", "u");
+const _reLetter = new RegExp("\\p{L}", "u");
+const getTextDirection = function(text) {
+    for (const l of text) {
+        if (l.match(_reLetterRtl))
+            return Clutter.TextDirection.RTL;
+        if (l.match(_reLetter))
+            return Clutter.TextDirection.LTR;
+    }
+    return Clutter.TextDirection.None;
+}
+
 // returns [x1,x2] so that the area between x1 and x2 is
 // centered in length
 
@@ -85,9 +99,11 @@ class AppGroup {
 
         this.signals = new SignalManager(null);
         this.appKeyTimeout = 0;
+        this.flashTimer = 0;
 
         // TODO: This needs to be in state so it can be updated more reliably.
-        this.labelVisible = this.state.settings.titleDisplay !== TitleDisplay.None && this.state.isHorizontal;
+        this.labelVisiblePref = this.state.settings.titleDisplay !== TitleDisplay.None && this.state.isHorizontal;
+        this.drawLabel = this.labelVisiblePref;
         this.progress = 0;
 
         this.actor =  new Cinnamon.GenericContainer({
@@ -110,27 +126,22 @@ class AppGroup {
         this.actor.add_child(this.progressOverlay);
 
         // Create the app button icon, number label, and text label for titleDisplay
-        this.iconBox = new St.Bin({name: 'appMenuIcon'});
+        this.iconBox = new Cinnamon.Slicer({name: 'appMenuIcon'});
         this.actor.add_child(this.iconBox);
         this.setActorAttributes(null, params.metaWindow);
 
         this.badge = new St.BoxLayout({
             style_class: 'grouped-window-list-badge',
             important: true,
-            width: 12 * global.ui_scale,
-            height: 12 * global.ui_scale,
             x_align: St.Align.START,
             y_align: St.Align.MIDDLE,
             show_on_set_parent: false,
-            style: 'margin: 0;',
         });
         this.numberLabel = new St.Label({
-            style: 'font-size: 10px;padding: 0px;',
             style_class: 'grouped-window-list-number-label',
             important: true,
             text: '',
             anchor_x: -3 * global.ui_scale,
-            anchor_y: 1 + (global.ui_scale > 1 ? 2 : 0)
         });
         this.numberLabel.clutter_text.ellipsize = false;
         this.badge.add(this.numberLabel, {
@@ -138,14 +149,15 @@ class AppGroup {
             y_align: St.Align.START,
         });
         this.actor.add_child(this.badge);
+        this.badge.set_text_direction(St.TextDirection.LTR);
 
         this.label = new St.Label({
             style_class: 'grouped-window-list-button-label',
             important: true,
             text: '',
+            x_align: St.Align.START,
             show_on_set_parent: this.state.settings.titleDisplay > 1
         });
-        this.label.x_align = St.Align.START;
         this.actor.add_child(this.label);
 
         this.groupState.set({tooltip: new Tooltips.PanelItemTooltip({actor: this.actor}, '', this.state.orientation)});
@@ -233,20 +245,10 @@ class AppGroup {
 
         let panelHeight = this.state.trigger('getPanelHeight');
 
-        // TODO: Button width should be applied to buttons if they don't have a label set, not based on
-        // mode, but not currently sure how to unset the fixed width on the actor so it revert to a
-        // resizable state without destroying it. Otherwise, buttons with labels don't have enough padding set.
-        if (!this.state.isHorizontal
-            || this.state.settings.titleDisplay === 1
-            || this.state.settings.titleDisplay === 3 && !this.labelVisible) {
-            this.actor.width = panelHeight;
-        }
-
         if (this.state.isHorizontal) {
             this.actor.height = panelHeight;
         }
         this.setIcon(metaWindow);
-        this.updateIconBoxClip();
         this.setIconPadding(panelHeight);
         this.setMargin();
     }
@@ -258,24 +260,14 @@ class AppGroup {
     }
 
     setMargin() {
-        let direction = this.state.isHorizontal ? 'right' : 'bottom';
-        let existingStyle = this.actor.style ? this.actor.style : '';
-        this.actor.style = existingStyle + 'margin-' + direction + ':6px;';
-    }
-
-    updateIconBoxClip() {
-        let iconBottomClip = this.iconBox.style_length('app-icon-bottom-clip');
-        let allocation = this.iconBox.allocation;
-        if (iconBottomClip > 0) {
-            this.iconBox.set_clip(
-                0,
-                0,
-                allocation.x2 - allocation.x1,
-                allocation.y2 - allocation.y1 - iconBottomClip
-            );
-        } else {
-            this.iconBox.remove_clip();
+        const applet = this.state.appletActor;
+        const direction = this.state.isHorizontal ? 'right' : 'bottom';
+        const existingStyle = this.actor.style ? this.actor.style : '';
+        let spacing = parseInt(applet.get_theme_node().get_length('spacing'));
+        if (!spacing) {
+            spacing = 6;
         }
+        this.actor.style = existingStyle + 'margin-' + direction + ':' + spacing + 'px;';
     }
 
     setIcon(metaWindow) {
@@ -313,32 +305,40 @@ class AppGroup {
         if (this._needsAttention) return;
 
         this._needsAttention = true;
-        let counter = 0;
-        this.flashButton(counter);
+        this.flashButton();
     }
 
-    flashButton(counter) {
-        if (!this._needsAttention || !this.actor) return;
-
-        // If the app was closed during a flash sequence, stop looping.
-        if (!this.groupState.groupReady && this.groupState.isFavoriteApp) {
-            this.actor.remove_style_class_name('grouped-window-list-item-demands-attention');
+    flashButton() {
+        if (!this._needsAttention || !this.actor || this.flashTimer)
             return;
-        }
 
-        this.actor.remove_style_pseudo_class('active');
-        this.actor.add_style_class_name('grouped-window-list-item-demands-attention');
-        if (counter < FLASH_MAX_COUNT) {
-            setTimeout(() => {
-                if (this.actor && this.actor.has_style_class_name('grouped-window-list-item-demands-attention')) {
-                    this.actor.remove_style_class_name('grouped-window-list-item-demands-attention');
-                    this.actor.add_style_pseudo_class('active');
-                }
-                setTimeout(() => {
-                    this.flashButton(++counter);
-                }, FLASH_INTERVAL);
-            }, FLASH_INTERVAL);
-        }
+        if (!this.groupState.groupReady && this.groupState.isFavoriteApp)
+            return;
+
+        let counter = 0;
+        const sc = "grouped-window-list-item-demands-attention";
+
+        this.flashTimer = Mainloop.timeout_add(FLASH_INTERVAL, () => {
+            if (!this._needsAttention) {
+                this.flashTimer = 0;
+                return GLib.SOURCE_REMOVE;
+            }
+
+            if (this.actor.has_style_class_name(sc)) {
+                this.actor.add_style_class_name("active");
+                this.actor.remove_style_class_name(sc);
+            }
+            else {
+                this.actor.remove_style_class_name("active")
+                this.actor.add_style_class_name(sc);
+            }
+
+            const continueFlashing = (counter++ < FLASH_MAX_COUNT);
+            if (!continueFlashing) {
+                this.flashTimer = 0;
+            }
+            return continueFlashing;
+        });
     }
 
     getPreferredWidth(actor, forHeight, alloc) {
@@ -346,11 +346,24 @@ class AppGroup {
         let [labelMinSize, labelNaturalSize] = this.label.get_preferred_width(forHeight);
         // The label text starts in the center of the icon, so we should allocate the space
         // needed for the icon plus the space needed for(label - icon/2)
-        alloc.min_size = iconNaturalSize + 6;
+        alloc.min_size = 1 * global.ui_scale;
+
+        let {appId, metaWindows, lastFocused} = this.groupState;
+
+        let allocateForLabel = false;
+
+        allocateForLabel = this.labelVisiblePref ||
+                           (this.state.settings.titleDisplay == TitleDisplay.Focused &&
+                            this.listState.lastFocusedApp === appId);
+
         if (this.state.orientation === St.Side.TOP || this.state.orientation === St.Side.BOTTOM) {
-            let max = this.labelVisible && this.groupState.metaWindows.length > 0 ?
-                labelNaturalSize + iconNaturalSize + 6 : 0;
-            alloc.natural_size = Math.min(iconNaturalSize + Math.max(max, labelNaturalSize), MAX_BUTTON_WIDTH * global.ui_scale);
+            if (allocateForLabel) {
+                let max = this.labelVisiblePref && this.groupState.metaWindows.length > 0 ?
+                    labelNaturalSize + iconNaturalSize + 6 : 0;
+                alloc.natural_size = Math.min(iconNaturalSize + Math.max(max, labelNaturalSize), MAX_BUTTON_WIDTH * global.ui_scale);
+            } else {
+                alloc.natural_size = iconNaturalSize + 6 * global.ui_scale;
+            }
         } else {
             alloc.natural_size = this.state.trigger('getPanelHeight');
         }
@@ -364,39 +377,34 @@ class AppGroup {
     }
 
     allocate(actor, box, flags) {
-        let allocWidth = box.x2 - box.x1;
-        let allocHeight = box.y2 - box.y1;
-        let childBox = new Clutter.ActorBox();
-        let direction = this.actor.get_text_direction();
-        let spacing = 0;
+        const allocWidth = box.x2 - box.x1;
+        const allocHeight = box.y2 - box.y1;
+        const childBox = new Clutter.ActorBox();
+        const direction = this.actor.get_text_direction();
 
         // Set the icon to be left-justified (or right-justified) and centered vertically
-        let [minWidth, minHeight, naturalWidth, naturalHeight] = this.iconBox.get_preferred_size();
-        let yPadding = Math.floor(Math.max(0, allocHeight - naturalHeight) / 2);
+        const [minWidth, minHeight, naturalWidth, naturalHeight] = this.iconBox.get_preferred_size();
+        const iconYPadding = Math.floor(Math.max(0, allocHeight - naturalHeight) / 2);
 
-        childBox.y1 = box.y1 + yPadding;
+        this.drawLabel = this.labelVisiblePref && allocWidth >= naturalWidth + 10 * global.ui_scale;
+
+        childBox.y1 = box.y1 + iconYPadding;
         childBox.y2 = childBox.y1 + Math.min(naturalHeight, allocHeight);
 
-        if (this.labelVisible && this.groupState.metaWindows.length > 0) {
-            if (direction === Clutter.TextDirection.LTR) {
-                childBox.x1 = box.x1 + 6;
-            } else {
-                childBox.x1 = Math.max(box.x1, box.x2 - naturalWidth);
-            }
-            spacing = childBox.x1;
+        if (this.drawLabel && this.groupState.metaWindows.length > 0) {
+            childBox.x1 = direction === Clutter.TextDirection.LTR ?
+                box.x1 + 6 : Math.max(box.x1, box.x2 - 6 - naturalWidth);
             childBox.x2 = Math.min(childBox.x1 + naturalWidth, box.x2);
         } else {
-            let offset = 0;
-            if (this.state.orientation === St.Side.LEFT) {
-                offset += this.actor.style_length('border-left-width') * 2;
-            }
+            const offset = this.state.orientation === St.Side.LEFT ? this.actor.style_length('border-left-width') * 2 : 0;
             [childBox.x1, childBox.x2] = center(allocWidth + offset, naturalWidth);
         }
 
         this.iconBox.allocate(childBox, flags);
 
-        let windowCountFactor = this.groupState.windowCount > 9 ? 1.5 : 2;
-        let badgeOffset = 2 * global.ui_scale;
+        // Set badge position
+        const windowCountFactor = this.groupState.windowCount > 9 ? 1.5 : 2;
+        const badgeOffset = 2 * global.ui_scale;
         childBox.x1 = childBox.x1 - badgeOffset;
         childBox.x2 = childBox.x1 + (this.numberLabel.width * windowCountFactor);
         childBox.y1 = Math.max(childBox.y1 - badgeOffset, 0);
@@ -404,23 +412,34 @@ class AppGroup {
 
         this.badge.allocate(childBox, flags);
 
-        if (this.labelVisible) {
-            let iconSize = this.state.trigger('getPanelIconSize') * global.ui_scale;
-            spacing = iconSize - spacing;
-            [minWidth, minHeight, naturalWidth, naturalHeight] = this.label.get_preferred_size();
+        // Set label position
+        if (this.drawLabel) {
+            const textDirection = getTextDirection(this.label.get_text());
+            const labelNaturalHeight = this.label.get_preferred_size()[3];
+            const labelYPadding = Math.floor(Math.max(0, allocHeight - labelNaturalHeight) / 2);
+
+            childBox.y1 = box.y1 + labelYPadding;
+            childBox.y2 = childBox.y1 + Math.min(labelNaturalHeight, allocHeight);
 
-            yPadding = Math.floor(Math.max(0, allocHeight - naturalHeight) / 2);
-            childBox.y1 = box.y1 + yPadding;
-            childBox.y2 = childBox.y1 + Math.min(naturalHeight, allocHeight);
             if (direction === Clutter.TextDirection.LTR) {
-                // Reuse the values from the previous allocation
-                childBox.x1 = Math.min(childBox.x2 + spacing, box.x2);
+                childBox.x1 = Math.min(this.iconBox.x + this.iconBox.width, box.x2);
                 childBox.x2 = box.x2;
             } else {
-                childBox.x2 = Math.max(childBox.x1 - spacing, box.x1);
                 childBox.x1 = box.x1;
+                childBox.x2 = this.iconBox.x;
             }
 
+            // Set text alignment
+            if (textDirection === St.TextDirection.LTR)
+                this.label.set_style('text-align: left;');
+            else if (textDirection === St.TextDirection.RTL)
+                this.label.set_style('text-align: right;');
+            else
+                if (direction === St.TextDirection.LTR)
+                    this.label.set_style('text-align: left;');
+                else
+                    this.label.set_style('text-align: right;');
+
             this.label.allocate(childBox, flags);
         }
 
@@ -429,7 +448,6 @@ class AppGroup {
             let rect = new Meta.Rectangle();
             [rect.x, rect.y] = this.actor.get_transformed_position();
             [rect.width, rect.height] = this.actor.get_transformed_size();
-
             each(this.groupState.metaWindows, (metaWindow) => {
                 if (metaWindow) {
                     metaWindow.set_icon_geometry(rect);
@@ -441,7 +459,7 @@ class AppGroup {
     }
 
     showLabel(animate = false) {
-        if (this.labelVisible
+        if (this.labelVisiblePref
             || !this.label
             || !this.state.isHorizontal
             || this.label.is_finalized()
@@ -451,12 +469,10 @@ class AppGroup {
 
         let width = MAX_BUTTON_WIDTH * global.ui_scale;
 
-        this.labelVisible = true;
+        this.labelVisiblePref = true;
         if (this.label.text == null) {
             this.label.set_text('');
         }
-        // TODO: This should be set by the theme.
-        this.label.set_style('padding-right: 4px;');
 
         if (!animate) {
             this.label.show();
@@ -480,7 +496,7 @@ class AppGroup {
         if (!this.label || this.label.is_finalized() || !this.label.realized) return;
 
         this.label.set_text('');
-        this.labelVisible = false;
+        this.labelVisiblePref = false;
         this.label.width = 1;
         this.label.hide();
     }
@@ -550,10 +566,15 @@ class AppGroup {
 
     allocateProgress(childBox = null, flags = 0) {
         if (!childBox) childBox = new Clutter.ActorBox();
-        childBox.x1 = 0;
         childBox.y1 = 0;
-        childBox.x2 = Math.max((this.actor.width) * (this.progress / 100.0), 1.0);
         childBox.y2 = this.actor.height;
+        if (St.Widget.get_default_direction() === St.TextDirection.RTL) {
+            childBox.x1 = Math.max(this.actor.width * ((100 - this.progress) / 100.0), 1.0);
+            childBox.x2 = this.actor.width;
+        } else {
+            childBox.x1 = 0;
+            childBox.x2 = Math.max(this.actor.width * (this.progress / 100.0), 1.0);
+        }
         this.progressOverlay.allocate(childBox, flags);
     }
 
@@ -582,9 +603,7 @@ class AppGroup {
         if (hasFocus) {
             this.listState.trigger('updateFocusState', appId);
             this.actor.add_style_pseudo_class('focus');
-            if (this.actor.has_style_class_name('grouped-window-list-item-demands-attention')) {
-                this.actor.remove_style_class_name('grouped-window-list-item-demands-attention');
-            }
+            this.actor.remove_style_class_name('grouped-window-list-item-demands-attention');
             this._needsAttention = false;
         } else {
             this.actor.remove_style_pseudo_class('focus');
@@ -794,7 +813,7 @@ class AppGroup {
             if (!this.rightClickMenu.isOpen) {
                 this.listState.trigger('closeAllRightClickMenus', () => {
                     this.listState.trigger('closeAllHoverMenus', () => {
-                        this.rightClickMenu.open();
+                        this.rightClickMenu.toggle();
                     });
                 });
             } else {
@@ -881,8 +900,9 @@ class AppGroup {
             this.signals.connect(metaWindow, 'notify::appears-focused', (...args) => this.onFocusWindowChange(...args));
             this.signals.connect(metaWindow, 'notify::gtk-application-id', (w) => this.onAppChange(w));
             this.signals.connect(metaWindow, 'notify::wm-class', (w) => this.onAppChange(w));
+            this.signals.connect(metaWindow, 'unmanaged', (w) => this.onAppChange(w));
 
-            this.signals.connect(metaWindow, 'icon-changed', (w) => this.setIcon(w));
+            this.signals.connect(metaWindow, 'notify::icon', (w) => this.setIcon(w));
 
             if (metaWindow.progress !== undefined) {
                 // Check if GWL is starting with pre-existing windows that have progress,
@@ -1085,7 +1105,6 @@ class AppGroup {
     handleTitleDisplayChange() {
         each(this.groupState.metaWindows, (win) => {
             this.onWindowTitleChanged(win, true);
-            this.handleButtonLabel(win);
         });
     }
 
@@ -1131,13 +1150,23 @@ class AppGroup {
         this.signals.disconnectAllSignals();
         this.groupState.set({willUnmount: true});
 
+        if (this.flashTimer > 0) {
+            Mainloop.source_remove(this.flashTimer);
+            this.flashTimer = 0;
+        }
+
         if (this.rightClickMenu) {
             if (this.rightClickMenu.isOpen) {
                 this.rightClickMenu.close();
             }
             this.rightClickMenu.destroy();
         }
-        if (this.hoverMenu) this.hoverMenu.destroy();
+
+        if (this.hoverMenu) {
+            Main.layoutManager.removeChrome(this.hoverMenu.actor);
+            this.hoverMenu.destroy();
+        }
+
         this.listState.trigger('removeChild', this.actor);
         this.actor.destroy();
 
