@@ -154,7 +154,7 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
         }
 
         // Preferences
-        let subMenu = new PopupMenu.PopupSubMenuMenuItem(_('Preferences'));
+        let subMenu = new PopupMenu.PopupSubMenuMenuItem(_('Applet preferences'));
         this.addMenuItem(subMenu);
 
         item = createMenuItem({label: _('About...'), icon: 'dialog-question'});
@@ -709,17 +709,34 @@ class WindowThumbnail {
         }
         if (this.metaWindowActor && !this.metaWindowActor.is_finalized()) {
             this.signals.connect(this.metaWindow, 'unmanaging', () => this.disconnectSizeNotify());
-            this.signals.connect(this.metaWindowActor, 'notify::size', () => this.refreshThumbnail());
+
+            let texture = this.metaWindowActor.get_texture();
+            if (texture == null) {
+                return;
+            }
+
+            this.signals.connect(texture, 'size-changed', () => this.refreshThumbnail());
 
             let [width, height] = this.metaWindowActor.get_size();
             let scale = Math.min(1.0, thumbnailWidth / width, thumbnailHeight / height) * global.ui_scale;
             width = Math.round(width * scale);
             height = Math.round(height * scale);
-            if (this.thumbnailActor.child) {
-                this.thumbnailActor.child.destroy()
-            }
+            if (this.thumbnailActor.child == null || (this.thumbnailActor.child.name?.startsWith("TextureWindowClone"))) {
+                if (this.thumbnailActor.child != null) {
+                    this.thumbnailActor.child.destroy()
+                }
 
-            this.thumbnailActor.child = WindowUtils.getCloneOrContent(this.metaWindowActor, width, height);
+                this.thumbnailActor.child = WindowUtils.getCloneOrContent(this.metaWindowActor, width, height);
+
+                if (this.thumbnailActor.child.name?.startsWith("TextureWindowClone")) {
+                    this.signals.connect(this.metaWindow, 'notify::minimized', () => this.refreshThumbnail());
+                }
+            } else {
+                this.thumbnailActor.child.width = width;
+                this.thumbnailActor.child.height = height;
+                this.thumbnailActor.queue_relayout();
+
+            }
         } else if (this.groupState.isFavoriteApp) {
             this.groupState.trigger('removeThumbnailFromMenu', this.metaWindow);
         }
@@ -727,7 +744,12 @@ class WindowThumbnail {
 
     disconnectSizeNotify(actor) {
         this.signals.disconnect('unmanaging', this.metaWindow);
-        this.signals.disconnect('notify::size', this.metaWindowActor);
+
+        let texture = this.metaWindowActor.get_texture();
+        if (texture) {
+            this.signals.disconnect("size-changed", texture);
+        }
+        this.signals.disconnect("notify::minimized", this.metaWindow);
     }
 
     refreshThumbnail() {
