@@ -298,6 +298,8 @@ class BrightnessSlider extends PopupMenu.PopupSliderMenuItem {
             this.tooltipText += ": " + value + "%";
 
         this.tooltip.set_text(this.tooltipText);
+        if (this._dragging)
+            this.tooltip.show();
     }
 
     /* Overriding PopupSliderMenuItem so we can modify the scroll step */
@@ -305,14 +307,13 @@ class BrightnessSlider extends PopupMenu.PopupSliderMenuItem {
         let direction = event.get_scroll_direction();
 
         if (direction == Clutter.ScrollDirection.DOWN) {
-            this._value = Math.max(0, this._value - this._step);
+            this._proxy.StepDownRemote(function() {});
         }
         else if (direction == Clutter.ScrollDirection.UP) {
-            this._value = Math.min(1, this._value + this._step);
+            this._proxy.StepUpRemote(function() {});
         }
 
         this._slider.queue_repaint();
-        this.emit('value-changed', this._value);
     }
 }
 
@@ -326,8 +327,8 @@ class CinnamonPowerApplet extends Applet.TextIconApplet {
 
         this.settings = new Settings.AppletSettings(this, metadata.uuid, instanceId);
 
-        Main.systrayManager.registerRole("power", metadata.uuid);
-        Main.systrayManager.registerRole("battery", metadata.uuid);
+        Main.systrayManager.registerTrayIconReplacement("power", metadata.uuid);
+        Main.systrayManager.registerTrayIconReplacement("battery", metadata.uuid);
 
         this.menuManager = new PopupMenu.PopupMenuManager(this);
         this.menu = new Applet.AppletPopupMenu(this, orientation);
@@ -342,7 +343,7 @@ class CinnamonPowerApplet extends Applet.TextIconApplet {
 
         this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
 
-        this.brightness = new BrightnessSlider(this, _("Brightness"), "display-brightness", BrightnessBusName, 0.01);
+        this.brightness = new BrightnessSlider(this, _("Brightness"), "display-brightness", BrightnessBusName, 0);
         this.keyboard = new BrightnessSlider(this, _("Keyboard backlight"), "keyboard-brightness", KeyboardBusName, 0);
         this.menu.addMenuItem(this.brightness);
         this.menu.addMenuItem(this.keyboard);
@@ -428,7 +429,7 @@ class CinnamonPowerApplet extends Applet.TextIconApplet {
                     status = ngettext("Charging - %d hour until fully charged", "Charging - %d hours until fully charged", hours).format(hours);
                 }
                 else {
-                    /* TRANSLATORS: this is a time string, as in "%d hours %d minutes remaining" */
+                    /* Translators: this is a time string, as in "%d hours %d minutes remaining" */
                     let template = _("Charging - %d %s %d %s until fully charged");
                     status = template.format (hours, ngettext("hour", "hours", hours), minutes, ngettext("minute", "minutes", minutes));
                 }
@@ -449,7 +450,7 @@ class CinnamonPowerApplet extends Applet.TextIconApplet {
                     status = ngettext("Using battery power - %d hour remaining", "Using battery power - %d hours remaining", hours).format(hours);
                 }
                 else {
-                    /* TRANSLATORS: this is a time string, as in "%d hours %d minutes remaining" */
+                    /* Translators: this is a time string, as in "%d hours %d minutes remaining" */
                     let template = _("Using battery power - %d %s %d %s remaining");
                     status = template.format (hours, ngettext("hour", "hours", hours), minutes, ngettext("minute", "minutes", minutes));
                 }
@@ -540,6 +541,7 @@ class CinnamonPowerApplet extends Applet.TextIconApplet {
                 this._deviceItems = [];
                 let devices_stats = [];
                 let pct_support_count = 0;
+                let _devices = []; let _deviceItems = [];
 
                 if (!error) {
                     let devices = result[0];
@@ -562,7 +564,7 @@ class CinnamonPowerApplet extends Applet.TextIconApplet {
 
                         let stats = "%s (%d%%)".format(deviceTypeToString(device_type), percentage);
                         devices_stats.push(stats);
-                        this._devices.push(devices[i]);
+                        _devices.push(devices[i]);
 
                         if (this._primaryDeviceId == null || this._primaryDeviceId == device_id) {
                             // Info for the primary battery (either the primary device, or any battery device if there is no primary device)
@@ -574,7 +576,7 @@ class CinnamonPowerApplet extends Applet.TextIconApplet {
                         let status = this._getDeviceStatus(devices[i]);
                         let item = new DeviceItem (devices[i], status, this.aliases);
                         this.menu.addMenuItem(item, position);
-                        this._deviceItems.push(item);
+                        _deviceItems.push(item);
                         position++;
                     }
                 }
@@ -582,6 +584,10 @@ class CinnamonPowerApplet extends Applet.TextIconApplet {
                     global.log(error);
                 }
 
+
+                this._devices = _devices;
+                this._deviceItems = _deviceItems;
+
                 // The menu is built. Below, we update the information present in the panel (icon, tooltip and label)
                 this.set_applet_enabled(true);
                 let panel_device = null;
@@ -685,7 +691,7 @@ class CinnamonPowerApplet extends Applet.TextIconApplet {
     }
 
     on_applet_removed_from_panel() {
-        Main.systrayManager.unregisterId(this.metadata.uuid);
+        Main.systrayManager.unregisterTrayIconReplacement(this.metadata.uuid);
     }
 }
 
