@@ -81,7 +81,7 @@ class LookingGlassProxy:
                 return self._proxy.GetResults('()')
             except Exception:
                 pass
-        return (False, "")
+        return False, ""
 
     def AddResult(self, code):
         if self._proxy:
@@ -96,7 +96,7 @@ class LookingGlassProxy:
                 return self._proxy.GetErrorStack('()')
             except Exception:
                 pass
-        return (False, "")
+        return False, ""
 
     def GetMemoryInfo(self):
         if self._proxy:
@@ -104,7 +104,7 @@ class LookingGlassProxy:
                 return self._proxy.GetMemoryInfo('()')
             except Exception:
                 pass
-        return (False, 0, {})
+        return False, 0, {}
 
     def FullGc(self):
         if self._proxy:
@@ -119,7 +119,7 @@ class LookingGlassProxy:
                 return self._proxy.Inspect('(s)', code)
             except Exception:
                 pass
-        return (False, "")
+        return False, ""
 
     def GetLatestWindowList(self):
         if self._proxy:
@@ -127,7 +127,7 @@ class LookingGlassProxy:
                 return self._proxy.GetLatestWindowList('()')
             except Exception:
                 pass
-        return (False, "")
+        return False, ""
 
     def StartInspector(self):
         if self._proxy:
@@ -142,7 +142,7 @@ class LookingGlassProxy:
                 return self._proxy.GetExtensionList('()')
             except Exception:
                 pass
-        return (False, "")
+        return False, ""
 
     def ReloadExtension(self, uuid, xlet_type):
         if self._proxy:
@@ -150,4 +150,4 @@ class LookingGlassProxy:
                 return self._proxy.ReloadExtension('(ss)', uuid, xlet_type)
             except Exception:
                 pass
-        return (False, "")
+        return False, ""
