@@ -21,7 +21,6 @@ const Util = imports.misc.util;
 const AppletManager = imports.ui.appletManager;
 
 var ANIMATION_TIME = 0.2;
-var NOTIFICATION_TIMEOUT = 4;
 var NOTIFICATION_CRITICAL_TIMEOUT_WITH_APPLET = 10;
 var SUMMARY_TIMEOUT = 1;
 var LONGER_SUMMARY_TIMEOUT = 4;
@@ -242,6 +241,9 @@ URLHighlighter.prototype = {
  */
 var Notification = class Notification {
     constructor(source, title, body, params) {
+        title = Util.decodeHTML(title);
+        body = Util.decodeHTML(body);
+
         this.source = source;
         this.title = title;
         this.urgency = Urgency.NORMAL;
@@ -400,8 +402,8 @@ var Notification = class Notification {
                  * many notifications.
                  */
                 this._scrollArea = new St.ScrollView({ name: 'notification-scrollview',
-                                                       vscrollbar_policy: Gtk.PolicyType.NEVER,
-                                                       hscrollbar_policy: Gtk.PolicyType.NEVER,
+                                                       vscrollbar_policy: St.PolicyType.NEVER,
+                                                       hscrollbar_policy: St.PolicyType.NEVER,
                                                        enable_mouse_scrolling: false/*,
                                                        style_class: 'vfade'*/ });
 
@@ -937,9 +939,9 @@ MessageTray.prototype = {
         this._notificationBin.opacity = 0;
 
         let monitor = Main.layoutManager.primaryMonitor;
-        let topPanel = Main.panelManager.getPanel(0, 0);
-        let bottomPanel = Main.panelManager.getPanel(0, 1);
-        let rightPanel = Main.panelManager.getPanel(0, 3);
+        let topPanel = Main.panelManager.getPanel(monitor.index, 0);
+        let bottomPanel = Main.panelManager.getPanel(monitor.index, 1);
+        let rightPanel = Main.panelManager.getPanel(monitor.index, 3);
         let topGap = 10;
         let bottomGap = 10;
         let rightGap = 0;
@@ -1012,16 +1014,17 @@ MessageTray.prototype = {
 
     _showNotificationCompleted: function() {
         this._updateNotificationTimeout(0);
+        this.notificationDuration = this.settings.get_int("notification-duration");
 
         if (this._notification.urgency != Urgency.CRITICAL) {
-            this._updateNotificationTimeout(NOTIFICATION_TIMEOUT * 1000);
+            this._updateNotificationTimeout(this.notificationDuration * 1000);
         } else if (AppletManager.get_role_provider_exists(AppletManager.Roles.NOTIFICATIONS)) {
             this._updateNotificationTimeout(NOTIFICATION_CRITICAL_TIMEOUT_WITH_APPLET * 1000);
         }
     },
 
     _updateNotificationTimeout: function(timeout) {
-        if (this._notificationTimeoutId) {
+        if (this._notificationTimeoutId > 0) {
             Mainloop.source_remove(this._notificationTimeoutId);
             this._notificationTimeoutId = 0;
         }
