@@ -330,7 +330,11 @@ var WindowManager = class WindowManager {
         // shown once. If they start minimized, monitor them until they've
         // been shown for the first time. (See windowUtils.js)
         const handleSeen = (metaWindow) => {
-            if (metaWindow === null || !Main.isInteresting(metaWindow)) {
+            if (this.windowSeen(metaWindow) || metaWindow === null || !Main.isInteresting(metaWindow)) {
+                return;
+            }
+
+            if (metaWindow.get_workspace().index() !== global.workspace_manager.get_active_workspace_index()) {
                 return;
             }
 
@@ -356,6 +360,11 @@ var WindowManager = class WindowManager {
             handleSeen(metaWindow);
         });
 
+        global.workspace_manager.connect("workspace-switched", (from, to, direction) => {
+            const allWindowActors = Meta.get_window_actors(global.display);
+            allWindowActors.forEach((actor) => handleSeen(actor.meta_window));
+        });
+
         const allWindowActors = Meta.get_window_actors(global.display);
         allWindowActors.forEach((actor) => handleSeen(actor.meta_window));
     }
@@ -861,6 +870,7 @@ var WindowManager = class WindowManager {
             {
                 actor.orig_opacity = actor.opacity;
                 actor.set_pivot_point(0.5, 0.5);
+                actor.x -= 1;
                 actor.scale_x = 0.94;
                 actor.scale_y = 0.94;
                 actor.opacity = 0;
@@ -872,6 +882,7 @@ var WindowManager = class WindowManager {
                     opacity: actor.orig_opacity,
                     scale_x: 1,
                     scale_y: 1,
+                    x: actor.x + 1,
                     duration: time * EASING_MULTIPLIER,
                     mode: Clutter.AnimationMode.EASE_OUT_QUAD,
                     onStopped: () => this._mapWindowDone(cinnamonwm, actor),
@@ -938,8 +949,6 @@ var WindowManager = class WindowManager {
             actor.set_pivot_point(0, 0);
             actor.scale_y = 1;
             actor.scale_x = 1;
-            actor.translation_y = 0;
-            actor.translation_x = 0;
             cinnamonwm.completed_map(actor);
         }
     }
