@@ -8,6 +8,7 @@
 #include <meta/util.h>
 #include <meta/meta-plugin.h>
 #include <meta/meta-shaped-texture.h>
+#include <meta/meta-cursor-tracker.h>
 
 #include "cinnamon-global.h"
 #include "cinnamon-screenshot.h"
@@ -27,6 +28,7 @@ struct _CinnamonScreenshot
 /* Used for async screenshot grabbing */
 typedef struct _screenshot_data {
   CinnamonScreenshot  *screenshot;
+  MetaWindow *window;
 
   char *filename;
 
@@ -34,6 +36,7 @@ typedef struct _screenshot_data {
   cairo_rectangle_int_t screenshot_area;
 
   gboolean include_cursor;
+  gboolean include_frame;
 
   CinnamonScreenshotCallback callback;
 } _screenshot_data;
@@ -83,11 +86,12 @@ write_screenshot_thread (GSimpleAsyncResult *result,
 }
 
 static void
-do_grab_screenshot (_screenshot_data *screenshot_data,
-                    int               x,
-                    int               y,
-                    int               width,
-                    int               height)
+do_grab_screenshot (_screenshot_data    *screenshot_data,
+                    ClutterPaintContext *paint_context,
+                    int                  x,
+                    int                  y,
+                    int                  width,
+                    int                  height)
 {
   CoglBitmap *bitmap;
   ClutterBackend *backend;
@@ -113,7 +117,8 @@ do_grab_screenshot (_screenshot_data *screenshot_data,
                                      CLUTTER_CAIRO_FORMAT_ARGB32,
                                      stride,
                                      data);
-  cogl_framebuffer_read_pixels_into_bitmap (cogl_get_draw_framebuffer (),
+
+  cogl_framebuffer_read_pixels_into_bitmap (clutter_paint_context_get_framebuffer (paint_context),
                                             x, y,
                                             COGL_READ_PIXELS_COLOR_BUFFER,
                                             bitmap);
@@ -123,73 +128,95 @@ do_grab_screenshot (_screenshot_data *screenshot_data,
 }
 
 static void
-_draw_cursor_image (cairo_surface_t *surface,
-                    cairo_rectangle_int_t area)
+_draw_cursor_image (cairo_surface_t       *surface,
+                   cairo_rectangle_int_t  area)
 {
-  XFixesCursorImage *cursor_image;
-
+  CoglTexture *texture;
+  int width, height;
+  int stride;
+  guint8 *data;
+  MetaDisplay *display;
+  MetaCursorTracker *tracker;
   cairo_surface_t *cursor_surface;
   cairo_region_t *screenshot_region;
   cairo_t *cr;
+  int x, y;
+  int xhot, yhot;
+  double xscale, yscale;
 
-  guchar *data;
-  int stride;
-  int i, j;
+  display = cinnamon_global_get_display (cinnamon_global_get ());
+  tracker = meta_cursor_tracker_get_for_display (display);
+  texture = meta_cursor_tracker_get_sprite (tracker);
 
-  cursor_image = XFixesGetCursorImage (clutter_x11_get_default_display ());
-
-  if (!cursor_image)
+  if (!texture)
     return;
 
   screenshot_region = cairo_region_create_rectangle (&area);
+  meta_cursor_tracker_get_pointer (tracker, &x, &y, NULL);
 
-  if (!cairo_region_contains_point (screenshot_region, cursor_image->x, cursor_image->y))
+  if (!cairo_region_contains_point (screenshot_region, x, y))
     {
-       XFree (cursor_image);
-       cairo_region_destroy (screenshot_region);
-       return;
+      cairo_region_destroy (screenshot_region);
+      return;
     }
 
-  cursor_surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, cursor_image->width, cursor_image->height);
+  meta_cursor_tracker_get_hot (tracker, &xhot, &yhot);
+  width = cogl_texture_get_width (texture);
+  height = cogl_texture_get_height (texture);
+  stride = 4 * width;
+  data = g_new (guint8, stride * height);
+  cogl_texture_get_data (texture, CLUTTER_CAIRO_FORMAT_ARGB32, stride, data);
 
-  /* The pixel data (in typical Xlib breakage) is longs even on
-   * 64-bit platforms, so we have to data-convert there. For simplicity,
-   * just do it always
-   */
-  data = cairo_image_surface_get_data (cursor_surface);
-  stride = cairo_image_surface_get_stride (cursor_surface);
-  for (i = 0; i < cursor_image->height; i++)
-    for (j = 0; j < cursor_image->width; j++)
-      *(guint32 *)(data + i * stride + 4 * j) = cursor_image->pixels[i * cursor_image->width + j];
+  /* FIXME: cairo-gl? */
+  cursor_surface = cairo_image_surface_create_for_data (data,
+                                                        CAIRO_FORMAT_ARGB32,
+                                                        width, height,
+                                                        stride);
 
-  cairo_surface_mark_dirty (cursor_surface);
+  cairo_surface_get_device_scale (surface, &xscale, &yscale);
+
+  if (xscale != 1.0 || yscale != 1.0)
+    {
+      int monitor;
+      float monitor_scale;
+      MetaRectangle cursor_rect = {
+        .x = x, .y = y, .width = width, .height = height
+      };
+
+      monitor = meta_display_get_monitor_index_for_rect (display, &cursor_rect);
+      monitor_scale = meta_display_get_monitor_scale (display, monitor);
+
+      cairo_surface_set_device_scale (cursor_surface, monitor_scale, monitor_scale);
+    }
 
   cr = cairo_create (surface);
   cairo_set_source_surface (cr,
                             cursor_surface,
-                            cursor_image->x - cursor_image->xhot - area.x,
-                            cursor_image->y - cursor_image->yhot - area.y);
+                            x - xhot - area.x,
+                            y - yhot - area.y);
   cairo_paint (cr);
 
   cairo_destroy (cr);
   cairo_surface_destroy (cursor_surface);
   cairo_region_destroy (screenshot_region);
-  XFree (cursor_image);
+  g_free (data);
 }
 
 static void
-grab_screenshot (ClutterActor *stage,
-                 _screenshot_data *screenshot_data)
+grab_screenshot (ClutterActor        *stage,
+                 ClutterPaintContext *paint_context,
+                 _screenshot_data    *screenshot_data)
 {
-  MetaScreen *screen = cinnamon_global_get_screen (screenshot_data->screenshot->global);
-  int width, height;
+  MetaDisplay *display = cinnamon_global_get_display (screenshot_data->screenshot->global);
+  int width, height, n_monitors;
   GSimpleAsyncResult *result;
 
-  meta_screen_get_size (screen, &width, &height);
+  meta_display_get_size (display, &width, &height);
 
-  do_grab_screenshot (screenshot_data, 0, 0, width, height);
+  do_grab_screenshot (screenshot_data, paint_context, 0, 0, width, height);
 
-  if (meta_screen_get_n_monitors (screen) > 1)
+  n_monitors = meta_display_get_n_monitors (display);
+  if (n_monitors > 1)
     {
       cairo_region_t *screen_region = cairo_region_create ();
       cairo_region_t *stage_region;
@@ -198,9 +225,9 @@ grab_screenshot (ClutterActor *stage,
       int i;
       cairo_t *cr;
 
-      for (i = meta_screen_get_n_monitors (screen) - 1; i >= 0; i--)
+      for (i = n_monitors - 1; i >= 0; i--)
         {
-          meta_screen_get_monitor_geometry (screen, i, &monitor_rect);
+          meta_display_get_monitor_geometry (display, i, &monitor_rect);
           cairo_region_union_rectangle (screen_region, (const cairo_rectangle_int_t *) &monitor_rect);
         }
 
@@ -237,6 +264,8 @@ grab_screenshot (ClutterActor *stage,
 
   g_signal_handlers_disconnect_by_func (stage, (void *)grab_screenshot, (gpointer)screenshot_data);
 
+  meta_enable_unredirect_for_display (display);
+
   result = g_simple_async_result_new (NULL, on_screenshot_written, (gpointer)screenshot_data, grab_screenshot);
   g_simple_async_result_run_in_thread (result, write_screenshot_thread, G_PRIORITY_DEFAULT, NULL);
   g_object_unref (result);
@@ -244,11 +273,14 @@ grab_screenshot (ClutterActor *stage,
 
 static void
 grab_area_screenshot (ClutterActor *stage,
+                      ClutterPaintContext *paint_context,
                       _screenshot_data *screenshot_data)
 {
+  MetaDisplay *display = cinnamon_global_get_display (screenshot_data->screenshot->global);
   GSimpleAsyncResult *result;
 
   do_grab_screenshot (screenshot_data,
+                      paint_context,
                       screenshot_data->screenshot_area.x,
                       screenshot_data->screenshot_area.y,
                       screenshot_data->screenshot_area.width,
@@ -258,11 +290,71 @@ grab_area_screenshot (ClutterActor *stage,
     _draw_cursor_image (screenshot_data->image, screenshot_data->screenshot_area);
 
   g_signal_handlers_disconnect_by_func (stage, (void *)grab_area_screenshot, (gpointer)screenshot_data);
+
+  meta_enable_unredirect_for_display (display);
+
   result = g_simple_async_result_new (NULL, on_screenshot_written, (gpointer)screenshot_data, grab_area_screenshot);
   g_simple_async_result_run_in_thread (result, write_screenshot_thread, G_PRIORITY_DEFAULT, NULL);
   g_object_unref (result);
 }
 
+static void
+grab_window_screenshot (ClutterActor *stage,
+                        ClutterPaintContext *paint_context,
+                        _screenshot_data *screenshot_data)
+{
+  MetaDisplay *display = cinnamon_global_get_display (screenshot_data->screenshot->global);
+  GSimpleAsyncResult *result;
+  ClutterActor *window_actor;
+  gfloat actor_x, actor_y;
+  MetaShapedTexture *stex;
+  MetaRectangle rect;
+  cairo_rectangle_int_t clip;
+
+  g_return_if_fail (META_IS_WINDOW (screenshot_data->window));
+
+  window_actor = CLUTTER_ACTOR (meta_window_get_compositor_private (screenshot_data->window));
+  clutter_actor_get_position (window_actor, &actor_x, &actor_y);
+
+  if (screenshot_data->include_frame || !meta_window_get_frame (screenshot_data->window))
+    {
+      meta_window_get_frame_rect (screenshot_data->window, &rect);
+
+      screenshot_data->screenshot_area.x = rect.x;
+      screenshot_data->screenshot_area.y = rect.y;
+
+      clip.x = rect.x - (gint) actor_x;
+      clip.y = rect.y - (gint) actor_y;
+    }
+  else
+    {
+      meta_window_get_buffer_rect (screenshot_data->window, &rect);
+
+      screenshot_data->screenshot_area.x = (gint) actor_x + rect.x;
+      screenshot_data->screenshot_area.y = (gint) actor_y + rect.y;
+
+      clip.x = rect.x;
+      clip.y = rect.y;
+    }
+
+  clip.width = screenshot_data->screenshot_area.width = rect.width;
+  clip.height = screenshot_data->screenshot_area.height = rect.height;
+
+  stex = META_SHAPED_TEXTURE (meta_window_actor_get_texture (META_WINDOW_ACTOR (window_actor)));
+  screenshot_data->image = meta_shaped_texture_get_image (stex, &clip);
+
+  if (screenshot_data->include_cursor)
+    _draw_cursor_image (screenshot_data->image, screenshot_data->screenshot_area);
+
+  g_signal_handlers_disconnect_by_func (stage, (void *)grab_window_screenshot, (gpointer) screenshot_data);
+
+  meta_enable_unredirect_for_display (display);
+
+  result = g_simple_async_result_new (NULL, on_screenshot_written, (gpointer)screenshot_data, cinnamon_screenshot_screenshot_window);
+  g_simple_async_result_run_in_thread (result, write_screenshot_thread, G_PRIORITY_DEFAULT, NULL);
+  g_object_unref (result);
+}
+
 /**
  * cinnamon_screenshot_screenshot:
  * @screenshot: the #CinnamonScreenshot
@@ -281,6 +373,7 @@ cinnamon_screenshot_screenshot (CinnamonScreenshot *screenshot,
                              const char *filename,
                              CinnamonScreenshotCallback callback)
 {
+  MetaDisplay *display;
   ClutterActor *stage;
   _screenshot_data *data = g_new0 (_screenshot_data, 1);
 
@@ -289,10 +382,11 @@ cinnamon_screenshot_screenshot (CinnamonScreenshot *screenshot,
   data->callback = callback;
   data->include_cursor = include_cursor;
 
+  display = cinnamon_global_get_display (screenshot->global);
   stage = CLUTTER_ACTOR (cinnamon_global_get_stage (screenshot->global));
 
+  meta_disable_unredirect_for_display (display);
   g_signal_connect_after (stage, "paint", G_CALLBACK (grab_screenshot), (gpointer)data);
-
   clutter_actor_queue_redraw (stage);
 }
 
@@ -321,6 +415,7 @@ cinnamon_screenshot_screenshot_area (CinnamonScreenshot *screenshot,
                                   const char *filename,
                                   CinnamonScreenshotCallback callback)
 {
+  MetaDisplay *display;
   ClutterActor *stage;
   _screenshot_data *data = g_new0 (_screenshot_data, 1);
 
@@ -333,8 +428,10 @@ cinnamon_screenshot_screenshot_area (CinnamonScreenshot *screenshot,
   data->callback = callback;
   data->include_cursor = include_cursor;
 
+  display = cinnamon_global_get_display (screenshot->global);
   stage = CLUTTER_ACTOR (cinnamon_global_get_stage (screenshot->global));
 
+  meta_disable_unredirect_for_display (display);
   g_signal_connect_after (stage, "paint", G_CALLBACK (grab_area_screenshot), (gpointer)data);
 
   clutter_actor_queue_redraw (stage);
@@ -360,11 +457,9 @@ cinnamon_screenshot_screenshot_window (CinnamonScreenshot *screenshot,
                                     const char *filename,
                                     CinnamonScreenshotCallback callback)
 {
-  GSimpleAsyncResult *result;
-
-  MetaScreen *screen = cinnamon_global_get_screen (screenshot->global);
-  MetaDisplay *display = meta_screen_get_display (screen);
+  MetaDisplay *display = cinnamon_global_get_display (screenshot->global);
   MetaWindow *window = meta_display_get_focus_window (display);
+  ClutterActor *stage;
 
   if (window == NULL || g_strcmp0 (meta_window_get_title (window), "Desktop") == 0)
   {
@@ -372,54 +467,22 @@ cinnamon_screenshot_screenshot_window (CinnamonScreenshot *screenshot,
     return;
   }
 
-  _screenshot_data *screenshot_data = g_new0 (_screenshot_data, 1);
+  _screenshot_data *data = g_new0 (_screenshot_data, 1);
 
-  ClutterActor *window_actor;
-  gfloat actor_x, actor_y;
-  MetaShapedTexture *stex;
-  MetaRectangle rect;
-  cairo_rectangle_int_t clip;
+  data->window = window;
+  data->screenshot = g_object_ref (screenshot);
+  data->filename = g_strdup (filename);
+  data->callback = callback;
+  data->include_cursor = include_cursor;
+  data->include_frame = include_frame;
 
-  screenshot_data->screenshot = g_object_ref (screenshot);
-  screenshot_data->filename = g_strdup (filename);
-  screenshot_data->callback = callback;
+  display = cinnamon_global_get_display (screenshot->global);
+  stage = CLUTTER_ACTOR (cinnamon_global_get_stage (screenshot->global));
 
-  window_actor = CLUTTER_ACTOR (meta_window_get_compositor_private (window));
-  clutter_actor_get_position (window_actor, &actor_x, &actor_y);
+  meta_disable_unredirect_for_display (display);
+  g_signal_connect_after (stage, "paint", G_CALLBACK (grab_window_screenshot), (gpointer) data);
 
-  if (include_frame || !meta_window_get_frame (window))
-    {
-      meta_window_get_outer_rect (window, &rect);
-
-      screenshot_data->screenshot_area.x = rect.x;
-      screenshot_data->screenshot_area.y = rect.y;
-
-      clip.x = rect.x - (gint) actor_x;
-      clip.y = rect.y - (gint) actor_y;
-    }
-  else
-    {
-      rect = *meta_window_get_rect (window);
-
-      screenshot_data->screenshot_area.x = (gint) actor_x + rect.x;
-      screenshot_data->screenshot_area.y = (gint) actor_y + rect.y;
-
-      clip.x = rect.x;
-      clip.y = rect.y;
-    }
-
-  clip.width = screenshot_data->screenshot_area.width = rect.width;
-  clip.height = screenshot_data->screenshot_area.height = rect.height;
-
-  stex = META_SHAPED_TEXTURE (meta_window_actor_get_texture (META_WINDOW_ACTOR (window_actor)));
-  screenshot_data->image = meta_shaped_texture_get_image (stex, &clip);
-
-  if (include_cursor)
-    _draw_cursor_image (screenshot_data->image, screenshot_data->screenshot_area);
-
-  result = g_simple_async_result_new (NULL, on_screenshot_written, (gpointer)screenshot_data, cinnamon_screenshot_screenshot_window);
-  g_simple_async_result_run_in_thread (result, write_screenshot_thread, G_PRIORITY_DEFAULT, NULL);
-  g_object_unref (result);
+  clutter_actor_queue_redraw (stage);
 }
 
 CinnamonScreenshot *
