@@ -630,29 +630,6 @@ cinnamon_get_event_state (ClutterEvent *event)
   return state;
 }
 
-/**
- * cinnamon_write_soup_message_to_stream:
- * @stream: a #GOutputStream
- * @message: a #SoupMessage
- * @error: location to store GError
- *
- * Write a string to a GOutputStream as binary data. This is a
- * workaround for the lack of proper binary strings in GJS.
- */
-void
-cinnamon_write_soup_message_to_stream (GOutputStream *stream,
-                                    SoupMessage   *message,
-                                    GError       **error)
-{
-  SoupMessageBody *body;
-
-  body = message->response_body;
-
-  g_output_stream_write_all (stream,
-                             body->data, body->length,
-                             NULL, NULL, error);
-}
-
 /**
  * cinnamon_write_string_to_stream:
  * @stream: a #GOutputStream
@@ -1089,4 +1066,4 @@ cinnamon_get_gpu_offload_supported (void)
     }
 
     return offload_supported;
-}
\ No newline at end of file
+}
