@@ -1,5 +1,4 @@
 #include "st-background-effect.h"
-#include "st-cogl-wrapper.h"
 #define ST_BACKGROUND_EFFECT_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), ST_TYPE_BACKGROUND_EFFECT, StBackgroundEffectClass))
 #define ST_IS_BACKGROUND_EFFECT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), ST_TYPE_BACKGROUND_EFFECT))
 #define ST_BACKGROUND_EFFECT_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), ST_TYPE_BACKGROUND_EFFECT, StBackgroundEffectClass))
@@ -156,17 +155,17 @@ st_background_effect_pre_paint (ClutterEffect *effect)
 
             if (self->bg_texture != NULL)
               {
-                cogl_handle_unref (self->bg_texture);
+                cogl_object_unref (self->bg_texture);
                 self->bg_texture = NULL;
               }
 
-            self->bg_texture = st_cogl_texture_new_from_data_wrapper  (self->bg_width_i,
-                                                                       self->bg_height_i,
-                                                                       COGL_TEXTURE_NO_SLICING,
-                                                                       COGL_PIXEL_FORMAT_RGBA_8888_PRE,
-                                                                       COGL_PIXEL_FORMAT_RGBA_8888_PRE,
-                                                                       rowstride,
-                                                                       data);
+            self->bg_texture = st_cogl_texture_new_from_data (self->bg_width_i,
+                                                              self->bg_height_i,
+                                                              COGL_TEXTURE_NO_SLICING,
+                                                              COGL_PIXEL_FORMAT_RGBA_8888_PRE,
+                                                              COGL_PIXEL_FORMAT_RGBA_8888_PRE,
+                                                              rowstride,
+                                                              data);
 
             g_free (data);
 
@@ -183,7 +182,7 @@ st_background_effect_pre_paint (ClutterEffect *effect)
 
       fg_texture = clutter_offscreen_effect_get_texture (offscreen_effect);
 
-      if (fg_texture != COGL_INVALID_HANDLE)
+      if (fg_texture != NULL)
         {
           self->fg_width_i = cogl_texture_get_width (fg_texture);
           self->fg_height_i = cogl_texture_get_height (fg_texture);
@@ -231,13 +230,13 @@ st_background_effect_pre_paint (ClutterEffect *effect)
 
               if (self->bg_sub_texture != NULL)
                 {
-                  cogl_handle_unref (self->bg_sub_texture);
+                  cogl_object_unref (self->bg_sub_texture);
                   self->bg_sub_texture = NULL;
                 }
 
-              self->bg_sub_texture = st_cogl_texture_new_with_size_wrapper (self->bg_width_i, self->bg_height_i,
-                                                                            COGL_TEXTURE_NO_SLICING,
-                                                                            COGL_PIXEL_FORMAT_RGBA_8888_PRE);
+              self->bg_sub_texture = cogl_texture_new_with_size (self->bg_width_i, self->bg_height_i,
+                                                                 COGL_TEXTURE_NO_SLICING,
+                                                                 COGL_PIXEL_FORMAT_RGBA_8888_PRE);
 
               cogl_pipeline_set_layer_texture (self->pipeline0, 0, self->bg_texture);
 
@@ -304,7 +303,7 @@ st_background_effect_paint_target (ClutterOffscreenEffect *effect)
       cogl_rectangle (-1.0f, 1.0f, 1.0f, -1.0f);
       cogl_pop_source ();
       cogl_pop_framebuffer ();
-      cogl_handle_unref (vertical_FBO);
+      cogl_object_unref (vertical_FBO);
 
       cogl_pipeline_set_layer_texture (self->pipeline1, 0, self->bg_sub_texture);
       cogl_push_source (self->pipeline1);
@@ -377,19 +376,19 @@ st_background_effect_dispose (GObject *gobject)
 
   if (self->bg_texture != NULL)
     {
-      cogl_handle_unref (self->bg_texture);
+      cogl_object_unref (self->bg_texture);
       self->bg_texture = NULL;
     }
 
   if (self->bg_sub_texture != NULL)
     {
-      cogl_handle_unref (self->bg_sub_texture);
+      cogl_object_unref (self->bg_sub_texture);
       self->bg_sub_texture = NULL;
     }
 
   if (self->bg_bumpmap != NULL)
     {
-      cogl_handle_unref (self->bg_bumpmap);
+      cogl_object_unref (self->bg_bumpmap);
       self->bg_bumpmap = NULL;
     }
 
@@ -413,7 +412,7 @@ st_background_effect_set_property (GObject      *gobject,
 
       if (self->bg_bumpmap != NULL)
         {
-          cogl_handle_unref (self->bg_bumpmap);
+          cogl_object_unref (self->bg_bumpmap);
           self->bg_bumpmap = NULL;
         }
 
@@ -425,9 +424,10 @@ st_background_effect_set_property (GObject      *gobject,
       file = g_file_new_for_path (g_strdup (self->bumpmap_location));
       if (g_file_query_exists (file, NULL))
         {
-          self->bg_bumpmap = st_cogl_texture_new_from_file_wrapper (self->bumpmap_location,
-                                                                    COGL_TEXTURE_NO_SLICING,
-                                                                    COGL_PIXEL_FORMAT_RGBA_8888_PRE);
+          self->bg_bumpmap = st_cogl_texture_new_from_file (self->bumpmap_location,
+                                                            COGL_TEXTURE_NO_SLICING,
+                                                            COGL_PIXEL_FORMAT_RGBA_8888_PRE,
+                                                            NULL);
         }
 
       g_object_unref (file);
@@ -675,9 +675,10 @@ st_background_effect_init (StBackgroundEffect *self)
 
   self->bumpmap_location = "/usr/share/cinnamon/bumpmaps/frost.png";
 
-  self->bg_bumpmap = st_cogl_texture_new_from_file_wrapper (self->bumpmap_location,
-                                                            COGL_TEXTURE_NO_SLICING,
-                                                            COGL_PIXEL_FORMAT_RGBA_8888_PRE);
+  self->bg_bumpmap = st_cogl_texture_new_from_file (self->bumpmap_location,
+                                                    COGL_TEXTURE_NO_SLICING,
+                                                    COGL_PIXEL_FORMAT_RGBA_8888_PRE,
+                                                    NULL);
   if (self->bg_bumpmap != NULL)
     {
       self->bumptex_width_i = cogl_texture_get_width (self->bg_bumpmap);
